<?php 
class Grid{
	var $output = '';
	var $displaydate; //date to display on top of the calendar
	var $dow; //starting day of week
	var $numcells = 0; //number of cells including fillers in the beginning
	var $numrows = 5;
	var $cells = array(); //the array of cells;
	var $rows = array(); //array of rows;
	var $nextDateBlock; 
	var $lastDateBlock;
	var $charnum;
	var $events;
	var $month;
	var $year;
	var $dstart;
	function Grid($rss_feed, $month, $year, $charnum, $parsed=false) {
		$dstart = mktime(0, 0, 0, $month, 1, $year);
		$this->month = $month;
		$this->year = $year;
		$this->dstart = $dstart;
		$this->today = date('z Y');
		$this->charnum = $charnum;
		$this->setAdjacentDates($month, $year);
		$this->displaydate = date('M Y',$dstart);
		$this->numOfDays = date('t',$dstart);
		$this->dow = date('w',$dstart);	//day of week M - F
		$this->numcells = date('w',$dstart) + $this->numOfDays;
		if($this->numcells > 35)
			$this->numrows = 6;
		if(!$parsed) {
			$this->parseRSSFeed($rss_feed);//fix events information
		} else {
			$this->events = $rss_feed;
		}
		$this->createCells();
		$this->loadOccurrences();
		$this->populateRows();
	}
	function parseRSSFeed($feed, $lastChar = '') {
		//Surpress errors
		libxml_use_internal_errors(true);
		//try to parse xml
		try {
			$data = new SimpleXMLElement($feed);//in rss form currently
		} catch(Exception $e){
			//most likely failed due to utf8 encoding issues, we try and fix this below
			//at other times its because the description has invalid utf8 characters which cause CDATA issues, not sure how to fix
			$errors =libxml_get_errors();
			foreach($errors as $error){
				if($error->code == 9) {//invalid character, this is what we are fixing
					$error = explode('Bytes: ', trim($error->message));//Get the bytes from string
					if(!isset($error[1])) {//not same message format?
						$feed = mb_convert_encoding($feed, 'UTF-8', mb_detect_encoding($feed, 'UTF-8, ISO-8859-1', true));
						$feed = html_entity_decode($feed,ENT_NOQUOTES,'UTF-8'); 
					} else {
						$byteArray = explode(' ',$error[1]);
						foreach($byteArray as &$byte) {
							$byte = hexdec($byte);
						}
						$string = implode(array_map("chr", $byteArray));
						if($lastChar == $string) {
							$utf8string = '';//get rid of it?
						} else {
							$utf8string = utf8_encode($string); 
						}
						$feed = str_replace($string, $utf8string, $feed);
					}
					$this->parseRSSFeed($feed, $string);
					return;
				} else {
					//not sure what to do
					//var_dump($error);	
				}
			}
		}
		$events = array();
		if(isset($data->channel)) {
			if(isset($data->channel->item)) {
				//pre-parse data
				foreach($data->channel->item as $item) {
					$event = array();
					$event['description'] = (string)$item->description;//for now assume it is okay.
					$event['title'] = str_replace("\n", "<br />", strip_tags((string)$item->title, '<sup><sub>'));//No tags allowed except sup and sub
					$event['link'] = strip_tags((string)$item->link);//no tags alowed
					$link_data = explode('/', $event['link']);
					$link_length = count($link_data);
					$event['series'] = $link_data[$link_length-2];//under assumption we have correct data, if not it just crashes which is fine
					$event['day'] = $link_data[$link_length-1];//under assumption we have correct data, if not it just crashes which is fine
					$image_url = explode(':imgurl:', $event['description']);
					$start_time = explode(':s_timestamp:', $event['description']);
					$end_time = explode(':e_timestamp:', $event['description']);
					$description = explode('<!-- Obscure Data -->', $event['description']);
					//Sanitize all data
					$event['description'] = str_replace("\n", "<br />", trim(strip_tags($description[0],  '<br><sup><sub>')));//No tags allowed except br, sup, and sub
					//The discription from RSS feed has an additional date and br(due to a new line after the date) so remove this so it displays like on events calendar
					$event['description'] = substr($event['description'], 16);//10 from date, 6 from <br />
					//either array(3) or array(1)
					if(isset($image_url[1])) {
						$event['image'] = strip_tags($image_url[1]);
					}
					//all events have start and end time
					$event['start'] = (int)$start_time[1];
					$event['end'] = (int)$end_time[1];
					$event['all_day'] = (date('H:i:s',$event['start'])=='00:00:00')&&(date('H:i:s',$event['end'])=='23:59:00');
					$events[] = $event;
				}
			}
		}
		$this->events = $events;
	}
	
	function setAdjacentDates($month,$year){
		if($month == '12'){
			$this->nextDateBlock = 'month=01&amp;year='.($year+1);
			$this->lastDateBlock = 'month=11&amp;year='.$year;
		}
		else if($month == '01'){
			$this->nextDateBlock = 'month=02&amp;year='.$year;
			$this->lastDateBlock = 'month=12&amp;year='.($year-1);
		} else {
			$nm = $month + 1;
			$lm = $month - 1;
			if($nm < 10){
				$nm = "0".$nm;
			}
			if($lm < 10){
				$lm = "0".$lm;
			}
			$this->nextDateBlock = 'month='.$nm.'&amp;year='.$year;
			$this->lastDateBlock = 'month='.$lm.'&amp;year='.$year;
		}
	}
	
	function createCells(){
		for($i = 1; $i < $this->numrows * 7 + 1; $i++){
			$day = $i;
			$datareplace = ".:[$day]:.";
			if(($i-$this->dow) > 0 && $i < $this->numcells){
				$cell = '<td>'.$datareplace.'</td>';
			} else {
				$cell = '<td class="exterior">'.$datareplace.'</td>';
			}
			$this->cells[$i] = $cell;
			$in = $this->dstart + ($i-$this->dow-1)*86400;
			$this->cells_date[$i] = '<th'.(date('z Y', $in) === $this->today?' class="today"':'').'><span><abbr title="'.date('l', $in).'">'.date('D', $in).'</abbr>, <abbr title="'.date('F', $in).'">'.date('M', $in).'</abbr> '.date('j', $in).' '.date('Y', $in).'</span></th>';		
			
		}
		
	}
	function open_time_slot($time) {
		$r = '<ol class="event-list">';//events holder
		$r .= '<li><span class="event-time">'.$time.'</span>';//new time entry
		$r .= '<ol>';//event links/title go here
		return $r;
	}
	function close_time_slot() {
		return '</ol></li></ol>';
	}
	function open_day() {
		return '';
	}
	function close_day() {
		return '';
	}
	function loadOccurrences(){
		$obarr = array();
		$prev_day = -30;
		$prev_time = '';
		$check_next_day = false;
		//for each event, put events into same day (in an ol tag)
		//for each event in same day, group by time. (ol tags inside of the day's ol tag)
		foreach($this->events as $key=>$event) {
			$month = date('m', $event['start']);//month of event
			
			$day = date('j',$event['start']);//day of event
			if($event['all_day']) {
				$time = 'All Day';
			} else {
				$time = date('g:ia', $event['start']);//time of event
				$time = str_replace(":00", "", $time);//remove sec if not there
			}
			if($month != $this->month) {//not same month
				//correct "$day" to index correctly
				$num_days = date('t',$event['start']);
				$corrected_day = $day-$num_days;
				if(($corrected_day + $this->dow) <= 0) {
					$corrected_day = $day+$this->numOfDays;
				}
				$day = $corrected_day;
			}
			$day = (int)$day;
			if($day == $prev_day) {//same day, continue day_data
				if($time != $prev_time) {//not same time
					if(!empty($prev_time)) {//if not first time slot(meaning nothing to close)
						$obarr[$day+$this->dow] .= $this->close_time_slot();
					}
					//open new time slow
					$obarr[$day+$this->dow] .= $this->open_time_slot($time);
				}
				
			} else {
				//store previous day data
				if($prev_day != -30) {//if not first instance
					$obarr[$prev_day+$this->dow] .= $this->close_time_slot();
					$obarr[$prev_day+$this->dow] .= $this->close_day();
				}
				//open day
				//$obarr[$day+$this->dow] needs to be initialized it is assigned a day and time slot, or else there are Undefined offset error messages
				$obarr[$day+$this->dow] = '';
				$obarr[$day+$this->dow] .= $this->open_day();//start list for a day
				$obarr[$day+$this->dow] .= $this->open_time_slot($time);//start list for a day_data
			}
			//making sure to place each event, above handles where
			
			$id = 'id_'.$event['series'].'_'.str_replace('-','_',$event['day']);
			$obarr[$day+$this->dow] .= '<li><a href="'.$event['link'].'" class="event-link series-'.$event['series'].'" id="'.$id.'"><strong>'.htmlspecialchars($event['title']).'</strong></a></li>';
			
			$prev_day = $day;
			$prev_time = $time;
		}
		if(!empty($prev_day)) {
			$obarr[$prev_day+$this->dow] .= $this->close_time_slot();
			$obarr[$prev_day+$this->dow] .= $this->close_day();
		}
		foreach($obarr as $key => $occurrencebunch){
			$this->cells[$key] = str_replace('.:['.($key).']:.',$occurrencebunch,$this->cells[$key]);
		}
		
		foreach($this->cells as $key => $cell){
			$this->cells[$key] = preg_replace('/.:\[[0-9]*\]:./','&nbsp;',$cell);
		}
	}

	function populateRows(){
		$rowdex = -1;
		//The rows need to be initialized before the cells and cell dates are assigned, or else there are Undefined offset error messages
		foreach($this->cells as $key=>$cell){
			if($key%7 == 1){
				$rowdex++;
			}
			$this->rows[$rowdex] = '';
			$this->date_rows[$rowdex] = '';		
		}
		$rowdex = -1;
		foreach($this->cells as $key=>$cell){
			if($key%7 == 1){
				$rowdex++;
			}
			$this->rows[$rowdex] .= $cell;
			$this->date_rows[$rowdex] .= $this->cells_date[$key];
		}
	}

	function drawGrid(){
		?>
		<div id="vb" class="toolbar"> 
			<a href="?month=<?php echo date('m');?>&year=<?php echo date('Y');?>" class="btn" id="vb-today" title="Skip to Today">Today</a> 
			<ul class="btn-multi" id="vb-timespan"> 
				<li><a href="?<?php echo $this->lastDateBlock;?>" title="Back" class="icon mousePrefetch"><span class="glyphicons chevron-left"></span></a></li> 
				<li><a href="#" class="btn-toggle"><?php echo $this->displaydate;?></a></li> 
				<li><a href="?<?php echo $this->nextDateBlock;?>" title="Forward" class="icon mousePrefetch"><span class="glyphicons chevron-right"></span></a></li> 
			</ul> 
		</div>
        <table id="events-grid" class="month">
            <tbody>
            <?php foreach($this->rows as $key => $row){?>
                <tr>
                <?php echo $this->date_rows[$key];?>
                </tr>
                <tr>
                <?php echo $row;?>
                </tr>
            <?php } ?>
            </tbody>
		</table><?php
	}
}
?>